#include <u.h>
#include <libc.h>
#include <bio.h>
#include "pedump.h"

char *Direntries[] = {			// Image directory entries
	"exports",
	"imports",
	"resources",
	"exceptions",
	"security",
	"base relocation",
	"debug",
	"architecture/copywrite",	// reserved, must be zero
	"global pointer (MIPS)",	// size must be zero
	"thread local store",
	"load config",
	"bound imports",
	"import addresses",
	"delay imports",
	"CLR runtime header",
	"reserved"			// must be zero
};

Namval Symclass[] = {
	-1,	"func-end",
	0,	"none",
	1,	"auto",
	2,	"ext-def",		// External definition 
	3,	"static",
	4,	"register",
	5,	"ext-ref",		// External reference 
	6,	"label",
	7,	"undef-label",
	8,	"struct-member",
	9,	"func-arg",
	10,	"struct-tag",
	11,	"union-member",
	12,	"union-tag",
	13,	"typedef",
	14,	"undef-static",		// Undefined static
	15,	"enum-tag",
	16,	"enum-memebr",
	17,	"reg-param",
	18,	"bit-field",
	19,	"tent-ext",		// Tentative external definition
	20,	"static-load-lab",	// Static load time label
	21,	"ext-load-lab",		// External load time label
	27,	"vararg",
	100,	"block",
	101,	"func",
	102,	"struct-end",
	103,	"file",
	104,	"section",
	105,	"weak-extern",
	0,	nil
};

Namval Symbasetype[] = {
	0,	"none",		// no info or unknown 
	1,	"void",
	2,	"char",	
	3,	"short",
	4,	"int",
	5,	"long",
	6,	"float",
	7,	"double",
	8,	"struct",
	9,	"union",
	10,	"enum",
	11,	"enum-val",
	12,	"byte",
	13,	"word",
	14,	"uint",
	15,	"dword",
	0,	nil
};

Namval Symbase[] = {
	0,	"none",		// no info or unknown 
	1,	"void",
	2,	"char",	
	3,	"short",
	4,	"int",
	5,	"long",
	6,	"float",
	7,	"double",
	8,	"struct",
	9,	"union",
	10,	"enum",
	11,	"enum-val",
	12,	"byte",
	13,	"word",
	14,	"uint",
	15,	"dword",
	0,	nil
};

Namval Symcomp[] = {
	0,	"scalar",
	1,	"pointer",
	2,	"function",
	3,	"vector",
	0,	nil
};

Namval Dosmagic[] = {
	0x5a4d,		"DOS EXE",
	0x4d5a,		"old DOS EXE",
	0, nil
};

Namval Petype[] = {
	0x10b,		"PE 32bit",
	0x20b,		"PE 64bit",
	0x107,		"PE 32bit ROM image",		/* not supported yet */
	0, nil
};

Namval Subsystem[] = {	/* NT header subsystem compiled for */
	0,		"Unknown",
	1,		"device driver or native windows",
	2,		"Windows GUI",
	3,		"Windows command line",
	5,		"OS/2 Command line",
	7,		"POSIX command line",
	9,		"Windows CE GUI",
	10,		"EFI application",
	11,		"EFI boot service",
	12,		"EFI runtime driver",
	13,		"EFI ROM",
	14,		"XBOX",
	0, nil
};

Namval Resourcetype[] = {	/* resource section record type */
	1,		"cursor",
	2,		"bitmap",
	3,		"icon",
	4,		"menu",
	5,		"dialog",
	6,		"string",
	7,		"font directory",
	8,		"font",
	9,		"accelerator",
	10,		"RC data",
	11,		"message table",
	12,		"group cursor",
	14,		"group icon",
	16,		"version",
	17,		"dialog include",
	19,		"plug-n-play",
	20,		"VDX",
	21,		"animated cursor",
	22,		"animated icon",
	23,		"HTML",
	24,		"manifest",
	0, nil
};

Namval Cputype[] = {	/* image header CPU type */
	0x014c,		"386",
	0x014d,		"i860",
	0x0162,		"R3000",
	0x0166,		"R4000 (le)",
	0x0168,		"R10000",
	0x0169,		"MIPS LE WinCE v2",
	0x0183,		"AXP (old)",
	0x0184,		"AXP",
	0x01a2,		"SH3",
	0x01a3,		"SH3 DSP",
	0x01a6,		"SH4",
	0x01a8,		"SH5",
	0x01c0,		"ARM LE",
	0x01c2,		"Thumb",
	0x01d3,		"AM33",
	0x01f0,		"ppc LE",
	0x01f1,		"ppc + FPU",
	0x0200,		"IA64",
	0x0266,		"MIPS 16",
	0x0268,		"68k",
	0x0284,		"AXP 64-bit",
	0x0366,		"MIPS + FPU",
	0x0466,		"MIPS 16 + FPU",
	0x0ebc,		"EFI Byte Code",
	0x8664,		"AMD64",
	0x9041,		"M32R LE",
	0xc0ee,		"clr pure MSIL",
	0, nil
};

Namval Secchar[] = {	/* section header characteristics */
	0x00000001,	"reserved",
	0x00000002,	"rezerved",
	0x00000004,	"reserved",
	0x00000008,	"no pad",	// depricated, set to zero
	0x00000010,	"reserved",
	0x00000020,	".text",
	0x00000040,	".data",
	0x00000080,	".bss",
	0x00000100,	"reserved",
	0x00000200,	"directives",	// obj files only
	0x00000400,	"reserved",
	0x00000800,	"link remove",	// obj files only
	0x00001000,	"COMDAT",	// obj files only
	0x00008000,	"uses global ptr",
	0x00020000,	"reserved",
	0x00040000,	"reserved",
	0x00080000,	"reserved",

//	0x00100000,	"no alignment",
	0x00200000,	"16bit aligned",
	0x00300000,	"32bit aligned",
	0x00400000,	"64bit aligned",
	0x00500000,	"128bit aligned",
	0x00600000,	"32byte aligned",
	0x00700000,	"64byte aligned",
	0x00800000,	"128byte aligned",
	0x00900000,	"256byte aligned",
	0x00a00000,	"512byte aligned",
	0x00b00000,	"1024byte aligned",
	0x00c00000,	"2048byte aligned",
	0x00d00000,	"4096byte aligned",
	0x00e00000,	"8192byte aligned",

	0x01000000,	"extended reloc",
	0x02000000,	"discardable",
	0x04000000,	"not cachable",
	0x08000000,	"not pageable",
	0x10000000,	"shared",
	0x20000000,	"executable",
	0x40000000,	"read",
	0x80000000,	"write",
	0, nil
}; 

Namval Imgchar[] = {	/* image header characteristics */
	0x0001,		"no reloc info",
	0x0002,		"EXE",
	0x0004,		"no line nums",			// depricated
	0x0008,		"no local syms",		// depricated
	0x0010,		"agressive trim",		// obsolete
	0x0020,		"large pointers",
	0x0040,		"reserved",
	0x0080,		"little endian",		// depricated
	0x0100,		"32bit",
	0x0200,		"no debug info",
	0x0400,		"image on removable media",
	0x0400,		"image on network",
	0x1000,		"is a system file",
	0x2000,		"is a DLL",
	0x4000,		"uniprocessor only",
	0x8000,		"big endian",			// depricated
	0, nil
};

Namval Dllchar[] = {
	0x0001,		"reserved",		// must be zero
	0x0002,		"reserved",		// must be zero
	0x0004,		"reserved",		// must be zero
	0x0008,		"reserved",		// must be zero
	0x0040,		"can be relocated",
	0x0080,		"integrity checks enforced",
	0x0100,		"NX compatible",
	0x0200,		"don't isolate",
	0x0400,		"no SEH used",
	0x0800,		"don't bind",
	0x1000,		"reserved",		// must be zero
	0x2000,		"WDM driver",
	0x8000,		"terminal server aware",
	0, nil
};

Prfmt Doshdr[] = {
	Tab,	2,	 Dosmagic, "Dos magic",
	Dec,	2,	 nil,	"file length % 512",
	Dec,	2,	 nil,	"file length / 512",
	Dec,	2,	 nil,	"relocations",
	Dec,	2,	 nil,	"header size / 16",
	Dec,	2,	 nil,	"min bss + stack % 16",
	Dec,	2,	 nil,	"max bss + stack / 16",
	Hex,	2,	 nil,	"initial (relative) SS value",
	Hex,	2,	 nil,	"initial SP value",
	Dec,	2,	 nil,	"neg sum of file",
	Hex,	2,	 nil,	"initial IP value",
	Hex,	2,	 nil,	"initial (relative) CS value",
	Dec,	2,	 nil,	"offset of relocation table",
	Dec,	2,	 nil,	"overlay number",
	Skip,	8,	 nil,	"reserved 1",
	Dec,	2,	 nil,	"OEM identifier",
	Dec,	2,	 nil,	"OEM information",
	Skip,	20,	 nil,	"reserved 2 ",
	0,	0,	 nil, 	nil
};

Prfmt Imagehdr[] = {
	Tab,	2,	 Cputype, "CPU", 
	Dec,	2,	 nil,	"section count",
	Time,	4,	 nil,	"time stamp",
	Dec,	4,	 nil,	"pointer to symbol table",
	Dec,	4,	 nil,	"symbol count",
	Dec,	2,	 nil,	"optional header size",
	Mask,	2,	 Imgchar, "characteristics",
	0,	0,	 nil, 	nil
};

Prfmt Coffhdr32[] = {
	Tab,	2,	 Petype, "file type ",
	Dec,	1,	 nil,	"linker major",
	Dec,	1,	 nil,	"linker minor",
	Dec,	4,	 nil,	".text size",
	Dec,	4,	 nil,	".data size",
	Dec,	4,	 nil,	".bss size",
	Hex,	4,	 nil,	"entry address",	// may be zero in DLLs
	Hex,	4,	 nil,	".text base address",
	Hex,	4,	 nil,	".data base address",
	0,	0,	 nil, 	nil
};

Prfmt Coffhdr64[] = {
	Tab,	2,	 Petype, "file type ",
	Dec,	1,	 nil,	"linker major",
	Dec,	1,	 nil,	"linker minor",
	Dec,	4,	 nil,	".text size",
	Dec,	4,	 nil,	".data size",
	Dec,	4,	 nil,	".bss size",
	Hex,	4,	 nil,	"entry address",	// may be zero in DLLs
	Hex,	4,	 nil,	".text base address",
	0,	0,	 nil, 	nil
};

Prfmt Nthdr32[] = {
	Hex,	4,	 nil,	"image base",		// multiple of 64k (0x400000)
	Dec,	4,	 nil,	"section align",	// >= file alignment (4096)
	Dec,	4,	 nil,	"file align",		// power of 2, 2 - 64k (512)
	Dec,	2,	 nil,	"os major",
	Dec,	2,	 nil,	"os minor",
	Dec,	2,	 nil,	"user major",
	Dec,	2,	 nil,	"user minor",
	Dec,	2,	 nil,	"subsys major",
	Dec,	2,	 nil,	"subsys minor",
	Skip,	4,	 nil,	"win32 version",	// must be zero
	Dec,	4,	 nil,	"image size",		// size of whole image, rounded up to section align
	Dec,	4,	 nil,	"header size",		// size of all headers rounded up to section align
	Hex,	4,	 nil,	"checksum",
	Tab,	2,	 Subsystem, "subsystem",
	Mask,	2,	 Dllchar, "DLL characteristics",
	Dec,	4,	 nil,	"stack reserve size",
	Dec,	4,	 nil,	"stack commit size",
	Dec,	4,	 nil,	"heap reserve size",
	Dec,	4,	 nil,	"heap commit size",
	Hex,	4,	 nil,	"loader flags",		// must be zero
/* Note the data directory count is not here but read by the caller */
	0,	0,	 nil, 	nil
};

Prfmt Nthdr64[] = {
	Hex,	8,	 nil,	"image base",		// multiple of 64k (0x400000)
	Dec,	4,	 nil,	"section align",	// >= file alignment (4096)
	Dec,	4,	 nil,	"file align",		// power of 2, 2 - 64k (512)
	Dec,	2,	 nil,	"os major",
	Dec,	2,	 nil,	"os minor",
	Dec,	2,	 nil,	"user major",
	Dec,	2,	 nil,	"user minor",
	Dec,	2,	 nil,	"subsys major",
	Dec,	2,	 nil,	"subsys minor",
	Skip,	4,	 nil,	"win32 version",	// must be zero
	Dec,	4,	 nil,	"image size",		// size of whole image, rounded up to section align
	Dec,	4,	 nil,	"header size",		// size of all headers rounded up to section align
	Hex,	4,	 nil,	"checksum",
	Tab,	2,	 Subsystem, "subsystem",
	Mask,	2,	 Dllchar, "DLL characteristics",
	Dec,	8,	 nil,	"stack reserve size",
	Dec,	8,	 nil,	"stack commit size",
	Dec,	8,	 nil,	"heap reserve size",
	Dec,	8,	 nil,	"heap commit size",
	Hex,	4,	 nil,	"loader flags",		// must be zero
/* Note the data directory count is not here but read by the caller */
	0,	0,	 nil, 	nil
};


Prfmt Sechdr[] = {
	Str,	8,	nil,	"name",			// may have no terminating zero
	Dec,	4,	nil,	"virtual size",		// size when loaded into RAM
	Hex,	4,	nil,	"virtual address",	// address when loaded into RAM, if > sizeofrawdata the its zero padded
	Dec,	4,	nil,	"size of raw data",	// size for data, size roundes to file align for text, zxero for bss
	Hex,	4,	nil,	"pointer to raw data",	// ptr to text on file align bounder, data on 4byte boundry, zero for BSS
	Skip,	2,	nil,	"pointer to relocations",	// zero for executables
	Skip,	2,	nil,	"pointer to line numbers",	// depricated, set to zero
	Skip,	4,	nil,	"number of relocations",	// zero for executables
	Skip,	4,	nil,	"number of line numbers",	// depricated, set to zero
	Mask,	4,	Secchar, "characteristics",
	0,	0,	 nil, 	nil
};

