#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ctype.h>
#include "pedump.h"

vlong
rint(Pe *pe, int sz)
{
	int i, m;
	vlong n;

	if(sz > sizeof(n) || sz < 1){
		fprint(2, "rint: bad size %d\n", sz);
		abort();
	}

	n = 0;
	for(i = 0; i < sz*8; i += 8){
		if((m = Bgetc(pe->bp)) < 0)
			sysfatal("%s - unexpected EOF [rint(%d)]\n", pe->file, sz);
		n |= m << i;
	}
	return n;
}

int
rmem(Pe *pe, void *buf, int n)
{
	int m;

	if((m = Bread(pe->bp, buf, n)) < 0)
		sysfatal("%s - unexpected EOF [rmem(%d)]\n", pe->file, n);
	return m;
}

int
skip(Pe *pe, int n)
{
	int m;

	if((m = Bseek(pe->bp, n, 1)) < 0)
		sysfatal("%s - unexpected EOF [skip(%d)]\n", pe->file, n);
	return m;
}

char *
lookup(Namval *nv, int val)
{
	static char buf[8];

	for(; nv->str; nv++)
		if(nv->val == val)
			break;
	if(nv->str == nil){
		snprint(buf, sizeof(buf), "<%d>", val);
		return buf;
	}

	return nv->str;
}


void
pr(Pe *pe, Prfmt *f)
{
	vlong n;
	Namval *v;
	char buf[1024];

	//print("%-6lld ", Bseek(pe->bp, 0LL, 1));

	switch(f->type){
	case Dec:
		n = rint(pe, f->sz);
		Bprint(op, "%-36s %lld\n", f->name, n);
		break;
	case Hex:
		n = rint(pe, f->sz);
		Bprint(op, "%-36s 0x%llx\n", f->name, n);
		break;
	case Tab:
		n = rint(pe, f->sz);
		for(v = f->tab; v->str; v++)
			if(v->val == n)
				break;
		if(v->str)
			Bprint(op, "%-36s %s\n", f->name, v->str);
		else
			Bprint(op, "%-36s [%lld - unknown]\n", f->name, n);
		break;
	case Mask:
		n = rint(pe, f->sz);
		Bprint(op, "%-36s ", f->name);
		for(v = f->tab; v->str; v++)
			if(v->val & n){
				Bprint(op, "%s, ", v->str);
				n &= ~v->val;
			}
		if(n != 0)
			Bprint(op, "[0x%llx unknown bits]", n);
		Bprint(op, "\n");
		break;
	case Str:
		if(f->sz > sizeof(buf))
			sysfatal("%s - %d string too long\n", f->name, f->sz);
		memset(buf, 0, sizeof(buf));
		rmem(pe, buf, f->sz);
		Bprint(op, "%-36s %s\n", f->name, buf);
		break;
	case Time:
		n = rint(pe, f->sz);
		Bprint(op, "%-36s %s", f->name, ctime(n));
		break;
	case Skip:
		skip(pe, f->sz);
//		Bprint(op, "%-36s skip %d bytes\n", f->name, f->sz);
		break;
	default:
		sysfatal("%d - type unknown\n", f->type);
	}
}

void
xd(Pe *pe, int addr, int len)
{
	int i, n;
	uchar buf[16];

	while(len > 0){
		if((n = Bread(pe->bp, buf, sizeof(buf))) <= 0)
			sysfatal("xd: unexpected EOF - %r\n");

		Bprint(op, "%-6x ", addr);
		for(i = 0; i < n; i++)
			Bprint(op, "%02x ", buf[i]);
		for(i = n; i < 16; i++)
			Bprint(op, "   ");
		Bprint(op, "   ");
		for(i = 0; i < n; i++)
			if(isprint(buf[i]))
				Bprint(op, "%c", buf[i]);
			else
				Bprint(op, "%c", '.');
		Bprint(op, "\n");
		addr += n;
		len -= n;
	}
}
